from django.contrib import admin

from .models import Card


@admin.register(Card)
class CardAdmin(admin.ModelAdmin):
	list_display = (
		"__str__", "card_id", "description", "card_set", "rarity", "type",
		"card_class", "artist"
	)
	list_filter = (
		"collectible", "card_set",
		"card_class", "type", "rarity", "cost",
		"battlecry", "deathrattle", "inspire",
		"divine_shield", "taunt", "windfury",
		"overload", "spell_damage"
	)
	search_fields = (
		"name", "description", "card_id",
	)



from enum import IntEnum

from django.db import models
from django_intenum import IntEnumField


class AdventureMode(IntEnum):
	# From ADVENTURE_MODE.xml
	NORMAL = 1
	EXPERT = 2
	HEROIC = 3
	CLASS_CHALLENGE = 4


class Adventure(models.Model):
	note_desc = models.CharField(max_length=64)
	name = models.CharField(max_length=64)
	sort_order = models.PositiveIntegerField(default=0)
	leaving_soon = models.BooleanField(default=False)

	created = models.DateTimeField(auto_now_add=True)
	updated = models.DateTimeField(auto_now=True)
	build = models.PositiveIntegerField()

	dbf_columns = ["ID", "NOTE_DESC", "NAME", "SORT_ORDER", "LEAVING_SOON"]

	def __str__(self):
		return self.name or self.note_desc


class Wing(models.Model):
	note_desc = models.CharField(max_length=64)
	adventure = models.ForeignKey(Adventure, on_delete=models.PROTECT)
	sort_order = models.PositiveIntegerField()
	release = models.CharField(max_length=16)
	required_event = models.CharField(max_length=16)
	ownership_prereq_wing = models.ForeignKey("Wing", models.SET_NULL, null=True, blank=True)
	name = models.CharField(max_length=64)
	coming_soon_label = models.CharField(max_length=64)
	requires_label = models.CharField(max_length=64)

	created = models.DateTimeField(auto_now_add=True)
	updated = models.DateTimeField(auto_now=True)
	build = models.PositiveIntegerField()

	dbf_columns = [
		"ID", "NOTE_DESC", "ADVENTURE_ID", "SORT_ORDER", "RELEASE", "REQUIRED_EVENT",
		"OWNERSHIP_PREREQ_WING_ID", "NAME", "COMING_SOON_LABEL", "REQUIRES_LABEL",
	]

	def __str__(self):
		return self.name or self.note_desc


class Scenario(models.Model):
	note_desc = models.CharField(max_length=64)
	players = models.PositiveSmallIntegerField()
	player1_hero_card_id = models.IntegerField(null=True)
	player2_hero_card_id = models.IntegerField(null=True)
	is_tutorial = models.BooleanField(default=False)
	is_expert = models.BooleanField(default=False)
	is_coop = models.BooleanField(default=False)
	adventure = models.ForeignKey(Adventure, models.SET_NULL, null=True, blank=True)
	wing = models.ForeignKey(Wing, models.SET_NULL, null=True, blank=True)
	sort_order = models.PositiveIntegerField(default=0)
	mode = IntEnumField(enum=AdventureMode, default=0)
	client_player2_hero_card_id = models.IntegerField(null=True)
	name = models.CharField(max_length=64)
	description = models.TextField()
	opponent_name = models.CharField(max_length=64)
	completed_description = models.TextField()
	player1_deck_id = models.IntegerField(null=True)

	created = models.DateTimeField(auto_now_add=True)
	updated = models.DateTimeField(auto_now=True)
	build = models.PositiveIntegerField()

	dbf_columns = [
		"ID", "NOTE_DESC", "PLAYERS", "PLAYER1_HERO_CARD_ID", "IS_TUTORIAL",
		"IS_EXPERT", "IS_COOP", "ADVENTURE_ID", "WING_ID", "SORT_ORDER",
		("MODE_ID", "mode"), "CLIENT_PLAYER2_HERO_CARD_ID", "NAME", "DESCRIPTION",
		"OPPONENT_NAME", "COMPLETED_DESCRIPTION", "PLAYER1_DECK_ID",
	]

	def __str__(self):
		return self.name or self.note_desc



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-02 08:45
from __future__ import unicode_literals

import django.db.models.deletion
import django_intenum
from django.db import migrations, models

import django_hearthstone.scenarios.models


class Migration(migrations.Migration):

	initial = True

	dependencies = []

	operations = [
		migrations.CreateModel(
			name="Adventure",
			fields=[
				(
					"id", models.AutoField(
						auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
					)
				),
				("note_desc", models.CharField(max_length=64)),
				("name", models.CharField(max_length=64)),
				("sort_order", models.PositiveIntegerField(default=0)),
				("leaving_soon", models.BooleanField(default=False)),
				("created", models.DateTimeField(auto_now_add=True)),
				("updated", models.DateTimeField(auto_now=True)),
				("build", models.PositiveIntegerField()),
			],
		),
		migrations.CreateModel(
			name="Scenario",
			fields=[
				(
					"id", models.AutoField(
						auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
					)
				),
				("note_desc", models.CharField(max_length=64)),
				("players", models.PositiveSmallIntegerField()),
				("player1_hero_card_id", models.IntegerField(null=True)),
				("player2_hero_card_id", models.IntegerField(null=True)),
				("is_tutorial", models.BooleanField(default=False)),
				("is_expert", models.BooleanField(default=False)),
				("is_coop", models.BooleanField(default=False)),
				("sort_order", models.PositiveIntegerField(default=0)),
				(
					"mode", django_intenum.IntEnumField(
						default=0, enum=django_hearthstone.scenarios.models.AdventureMode
					)
				),
				("client_player2_hero_card_id", models.IntegerField(null=True)),
				("name", models.CharField(max_length=64)),
				("description", models.TextField()),
				("opponent_name", models.CharField(max_length=64)),
				("completed_description", models.TextField()),
				("player1_deck_id", models.IntegerField(null=True)),
				("created", models.DateTimeField(auto_now_add=True)),
				("updated", models.DateTimeField(auto_now=True)),
				("build", models.PositiveIntegerField()),
				(
					"adventure", models.ForeignKey(
						blank=True,
						null=True,
						on_delete=django.db.models.deletion.SET_NULL,
						to="scenarios.Adventure"
					)
				),
			],
		),
		migrations.CreateModel(
			name="Wing",
			fields=[
				(
					"id", models.AutoField(
						auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
					)
				),
				("note_desc", models.CharField(max_length=64)),
				("sort_order", models.PositiveIntegerField()),
				("release", models.CharField(max_length=16)),
				("required_event", models.CharField(max_length=16)),
				("name", models.CharField(max_length=64)),
				("coming_soon_label", models.CharField(max_length=64)),
				("requires_label", models.CharField(max_length=64)),
				("created", models.DateTimeField(auto_now_add=True)),
				("updated", models.DateTimeField(auto_now=True)),
				("build", models.PositiveIntegerField()),
				(
					"adventure", models.ForeignKey(
						on_delete=django.db.models.deletion.PROTECT, to="scenarios.Adventure"
					)
				),
				(
					"ownership_prereq_wing", models.ForeignKey(
						blank=True,
						null=True,
						on_delete=django.db.models.deletion.SET_NULL,
						to="scenarios.Wing"
					)
				),
			],
		),
		migrations.AddField(
			model_name="scenario",
			name="wing",
			field=models.ForeignKey(
				blank=True,
				null=True,
				on_delete=django.db.models.deletion.SET_NULL,
				to="scenarios.Wing"
			),
		),
	]



import os
from argparse import ArgumentTypeError
from collections import OrderedDict

from django.core.management.base import BaseCommand
from hearthstone.dbf import Dbf

from ...models import Adventure, Scenario, Wing


def build_range(value):
	value = int(value)
	if value < 0 or value > 2**32:
		raise ArgumentTypeError("Invalid build number: %r" % (value))
	return value


class Command(BaseCommand):
	tables = OrderedDict([
		("ADVENTURE", Adventure),
		("WING", Wing),
		("SCENARIO", Scenario),
	])

	def add_arguments(self, parser):
		parser.add_argument("path", nargs=1)
		parser.add_argument("--build", type=build_range, required=True)
		parser.add_argument("--force", action="store_true")
		parser.add_argument("--locale", default="enUS")

	def get_values(self, record, columns):
		values = {"build": self.build}
		for column in columns:
			if isinstance(column, tuple):
				column, field = column
			else:
				field = column.lower()
			value = record[column]
			if field.endswith("_id") and value == 0:
				value = None
			if value is not None:
				if isinstance(value, dict):
					value = value[self.locale]
				values[field] = value

		return values

	def load_dbf(self, filename):
		self.stdout.write("Parsing %s" % (filename))
		dbf = Dbf.load(filename)
		if dbf.name not in self.tables:
			self.stderr.write("No handler for %r (%r)" % (filename, dbf.name))
			return

		cls = self.tables[dbf.name]
		for record in dbf.records:
			values = self.get_values(record, cls.dbf_columns)
			try:
				instance = cls.objects.get(id=record["ID"])
			except cls.DoesNotExist:
				instance = cls.objects.create(**values)
				self.stdout.write("Created %r (build %r)" % (instance, self.build))
			else:
				if self.force or instance.build < self.build:
					self.stdout.write("Updating %r to build %r" % (instance, self.build))
					for k, v in values.items():
						setattr(instance, k, v)
					instance.save()
				else:
					self.stdout.write("Skipping %r (up to date)" % (instance))

	def load_dbf_folder(self, path):
		for dbf_name in self.tables:
			filename = os.path.join(path, dbf_name + ".xml")
			self.load_dbf(filename)

	def handle(self, *args, **options):
		path = options["path"][0]
		self.build = options["build"]
		self.force = options["force"]
		self.locale = options["locale"]

		if os.path.isdir(path):
			self.load_dbf_folder(path)
		else:
			self.load_dbf(path)



from django.core.management.base import BaseCommand
from django.db import transaction
from hearthstone import cardxml
from hearthstone.enums import Locale

from ...models import Card, CardString, CardTag


class Command(BaseCommand):
	def add_arguments(self, parser):
		parser.add_argument("path", nargs="?", help="CardDefs.xml file")
		parser.add_argument("--locale", default="enUS")
		parser.add_argument("--force", action="store_true", help="Force update all cards")

	def handle(self, *args, **options):
		path = options["path"]
		db, _ = cardxml.load_dbf(path, locale=options["locale"])
		self.stdout.write("%i cards available" % (len(db)))

		qs = Card.objects.all().values_list("dbf_id")
		known_ids = [item[0] for item in qs]
		missing = [id for id in db if id not in known_ids]
		self.stdout.write("%i known cards" % (len(known_ids)))

		new_cards = [Card.from_cardxml(db[id]) for id in missing]
		Card.objects.bulk_create(new_cards)
		self.stdout.write("%i new cards" % (len(new_cards)))

		if options["force"]:
			existing = Card.objects.filter(dbf_id__in=known_ids)
			for card in existing:
				if card.dbf_id not in db:
					self.stderr.write(
						"WARNING: %r (%s) not in CardDefs.xml. Skipping." % (card, card.dbf_id)
					)
					continue
				c = db[card.dbf_id]
				if c:
					card.update_from_cardxml(c, save=True)
			self.stdout.write("%i updated cards" % (len(existing)))

		tags = []
		for dbf_id, card in db.items():
			for tag, value in card.tags.items():
				tags.append(CardTag(card_id=dbf_id, game_tag=tag, value=value))

		locstrings = []
		for dbf_id, card in db.items():
			for tag, locstring in card.strings.items():
				if isinstance(locstring, str):
					# (non-localized string)
					continue
				for locale, value in locstring.items():
					locale = Locale[locale]
					locstrings.append(CardString(
						card_id=dbf_id, locale=locale, game_tag=tag, value=value
					))

		with transaction.atomic():
			CardTag.objects.all().delete()
			CardString.objects.all().delete()

			CardTag.objects.bulk_create(tags)
			CardString.objects.bulk_create(locstrings)



from django.db import models
from django.urls import reverse
from django.utils.text import slugify
from django_intenum import IntEnumField
from hearthstone import cardxml, enums


DBF_DB = {}


class IncludibleCardManager(models.Manager):
	def get_queryset(self):
		return super().get_queryset().filter(
			models.Q(collectible=True),
			models.Q(type__in=[
				enums.CardType.MINION,
				enums.CardType.SPELL,
				enums.CardType.WEAPON
			]) | (
				models.Q(type=enums.CardType.HERO) &
				~models.Q(card_set__in=[enums.CardSet.CORE, enums.CardSet.HERO_SKINS])
			)
		)


class Card(models.Model):
	card_id = models.CharField(primary_key=True, max_length=50)
	dbf_id = models.IntegerField(null=True, unique=True, db_index=True)

	name = models.CharField(max_length=64)
	description = models.TextField(blank=True)
	flavortext = models.TextField(blank=True)
	how_to_earn = models.TextField(blank=True)
	how_to_earn_golden = models.TextField(blank=True)
	artist = models.CharField(max_length=255, blank=True)

	card_class = IntEnumField(enum=enums.CardClass, default=enums.CardClass.INVALID)
	card_set = IntEnumField(enum=enums.CardSet, default=enums.CardSet.INVALID)
	faction = IntEnumField(enum=enums.Faction, default=enums.Faction.INVALID)
	race = IntEnumField(enum=enums.Race, default=enums.Race.INVALID)
	rarity = IntEnumField(enum=enums.Rarity, default=enums.Rarity.INVALID)
	type = IntEnumField(enum=enums.CardType, default=enums.CardType.INVALID)

	collectible = models.BooleanField(default=False)
	battlecry = models.BooleanField(default=False)
	divine_shield = models.BooleanField(default=False)
	deathrattle = models.BooleanField(default=False)
	elite = models.BooleanField(default=False)
	evil_glow = models.BooleanField(default=False)
	inspire = models.BooleanField(default=False)
	forgetful = models.BooleanField(default=False)
	one_turn_effect = models.BooleanField(default=False)
	poisonous = models.BooleanField(default=False)
	ritual = models.BooleanField(default=False)
	secret = models.BooleanField(default=False)
	taunt = models.BooleanField(default=False)
	topdeck = models.BooleanField(default=False)

	atk = models.IntegerField(default=0)
	health = models.IntegerField(default=0)
	durability = models.IntegerField(default=0)
	cost = models.IntegerField(default=0)
	windfury = models.IntegerField(default=0)

	spare_part = models.BooleanField(default=False)
	overload = models.IntegerField(default=0)
	spell_damage = models.IntegerField(default=0)

	craftable = models.BooleanField(default=False)

	objects = models.Manager()
	includibles = IncludibleCardManager()

	class Meta:
		db_table = "card"

	@property
	def id(self):
		return self.card_id

	@classmethod
	def from_cardxml(cls, card, save=False):
		obj = cls(card_id=card.id)
		obj.update_from_cardxml(card, save=save)
		return obj

	@classmethod
	def get_string_id(cls, dbf_id):
		if not DBF_DB:
			db, _ = cardxml.load_dbf()
			DBF_DB.update(db)
		return DBF_DB[dbf_id].id

	def __str__(self):
		return self.name

	@property
	def slug(self):
		return slugify(self.name)

	def get_absolute_url(self):
		# XXX
		return reverse("card_detail", kwargs={"pk": self.dbf_id, "slug": self.slug})

	def get_card_art_url(self, resolution=256, format="jpg"):
		return "https://art.hearthstonejson.com/v1/%ix/%s.%s" % (resolution, self.id, format)

	def get_card_render_url(self, resolution=256, format="png", locale="enUS", build="latest"):
		return "https://art.hearthstonejson.com/v1/render/%s/%s/%ix/%s.%s" % (
			build, locale, resolution, self.id, format
		)

	def get_tile_url(self, format="png"):
		return "https://art.hearthstonejson.com/v1/tiles/%s.%s" % (self.id, format)

	def localized_name(self, locale) -> str:
		try:
			return self.strings.get(locale=locale, game_tag=enums.GameTag.CARDNAME).value
		except CardString.DoesNotExist:
			return ""

	def update_from_cardxml(self, cardxml, save=False):
		for k in dir(cardxml):
			if k.startswith("_") or k in ("id", "tags", "strings"):
				continue
			# Transfer all existing CardXML attributes to our model
			if hasattr(self, k):
				setattr(self, k, getattr(cardxml, k))

		if save:
			self.save()

	@property
	def playable(self):
		"""Returns whether the card can be played."""
		if self.type in [enums.CardType.MINION, enums.CardType.SPELL, enums.CardType.WEAPON]:
			return True
		# Heroes can only be played if they are not the basic heroes or hero skins.
		if self.type == enums.CardType.HERO:
			return self.card_set not in [enums.CardSet.CORE, enums.CardSet.HERO_SKINS]
		return False

	@property
	def includible(self):
		return self.collectible and self.playable


class CardTag(models.Model):
	card = models.ForeignKey(
		"Card", to_field="dbf_id", db_column="card_dbf_id", related_name="tags",
		on_delete=models.CASCADE
	)
	game_tag = IntEnumField(enum=enums.GameTag, db_index=True)
	value = models.PositiveIntegerField()

	def __str__(self):
		return f"{str(self.card)}.{self.game_tag.name}={str(self.value)}"


class CardString(models.Model):
	card = models.ForeignKey(
		"Card", to_field="dbf_id", db_column="card_dbf_id", related_name="strings",
		on_delete=models.CASCADE
	)
	locale = IntEnumField(enum=enums.Locale, db_index=True)
	game_tag = IntEnumField(enum=enums.GameTag, db_index=True)
	value = models.TextField(blank=True)

	def __str__(self):
		return self.value



# Generated by Django 2.0.3 on 2018-03-21 11:13

import django.db.models.deletion
import django_intenum
import hearthstone.enums
from django.db import migrations, models


class Migration(migrations.Migration):

	dependencies = [
		("cards", "0001_initial"),
	]

	operations = [
		migrations.CreateModel(
			name="CardString",
			fields=[
				("id", models.AutoField(
					auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
				)),
				("locale", django_intenum.IntEnumField(db_index=True, enum=hearthstone.enums.Locale)),
				("game_tag", django_intenum.IntEnumField(
					db_index=True, enum=hearthstone.enums.GameTag
				)),
				("value", models.TextField(blank=True)),
			],
		),
		migrations.CreateModel(
			name="CardTag",
			fields=[
				("id", models.AutoField(
					auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
				)),
				("game_tag", django_intenum.IntEnumField(
					db_index=True, enum=hearthstone.enums.GameTag
				)),
				("value", models.PositiveIntegerField()),
			],
		),
		migrations.AlterField(
			model_name="card",
			name="card_class",
			field=django_intenum.IntEnumField(default=0, enum=hearthstone.enums.CardClass),
		),
		migrations.AlterField(
			model_name="card",
			name="card_set",
			field=django_intenum.IntEnumField(default=0, enum=hearthstone.enums.CardSet),
		),
		migrations.AlterField(
			model_name="card",
			name="faction",
			field=django_intenum.IntEnumField(default=0, enum=hearthstone.enums.Faction),
		),
		migrations.AlterField(
			model_name="card",
			name="race",
			field=django_intenum.IntEnumField(default=0, enum=hearthstone.enums.Race),
		),
		migrations.AlterField(
			model_name="card",
			name="rarity",
			field=django_intenum.IntEnumField(default=0, enum=hearthstone.enums.Rarity),
		),
		migrations.AlterField(
			model_name="card",
			name="type",
			field=django_intenum.IntEnumField(default=0, enum=hearthstone.enums.CardType),
		),
		migrations.AddField(
			model_name="cardtag",
			name="card",
			field=models.ForeignKey(
				on_delete=django.db.models.deletion.CASCADE, related_name="tags",
				to="cards.Card", to_field="dbf_id", db_column="card_dbf_id"
			),
		),
		migrations.AddField(
			model_name="cardstring",
			name="card",
			field=models.ForeignKey(
				on_delete=django.db.models.deletion.CASCADE, related_name="strings",
				to="cards.Card", to_field="dbf_id", db_column="card_dbf_id"
			),
		),
	]
